SystemOrganization addCategory: #'Neo-JSON-Tests'!

Object subclass: #NeoJSONBenchmark
	instanceVariableNames: 'objects json'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Tests'!

!NeoJSONBenchmark commentStamp: 'SvenVanCaekenberghe 6/5/2012 09:48' prior: 0!
I am NeoJSONBenchmark.
I hold 1000 objects of the form Dictionary('color'->'green' 'enabled'->false 'id'->1 'name'->'Name-1' 'value'->0.002 ) in an Array.
I hold the JSON serialization of that as a String of the form '[{"value":0.002,"enabled":false,"name":"Name-1","color":"green","id":1}, ...

| benchmark |
benchmark := NeoJSONBenchmark new.
[benchmark read] bench.

| benchmark |
benchmark := NeoJSONBenchmark new.
[benchmark write] bench.

| benchmark |
benchmark := NeoJSONBenchmark new.
[benchmark read] timeToRun.

| benchmark |
benchmark := NeoJSONBenchmark new.
[benchmark write] timeToRun.!

!NeoJSONBenchmark methodsFor: 'initialize-release' stamp: 'SvenVanCaekenberghe 6/4/2012 18:49'!
initialize
	super initialize.
	objects := Array new: 1000.
	1 to: 1000 do: [ :each |
		| map |
		(map := Dictionary new)
			at: 'id' put: each;
			at: 'name' put: 'Name-', each asString;
			at: 'enabled' put: (#( true false) at: (each \\ 2) + 1);
			at: 'color' put: (#( 'red' 'green' 'blue' ) at: (each \\ 3) + 1);
			at: 'value' put: each + each / 1000.0.
		objects at: each put: map ].
	json := String streamContents: [ :stream |
		(NeoJSONWriter on: stream) nextPut: objects ]
		! !

!NeoJSONBenchmark methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/4/2012 17:02'!
read
	^ (NeoJSONReader on: json readStream) next
! !

!NeoJSONBenchmark methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/4/2012 17:26'!
write
	^ String streamContents: [ :stream |
		(NeoJSONWriter on: stream) nextPut: objects ]
! !

!NeoJSONBenchmark methodsFor: 'public' stamp: 'SvenVanCaekenberghe 6/6/2012 15:07'!
writePrettyPrintedRead
	| output |
	output := String streamContents: [ :stream |
		(NeoJSONWriter on: stream) 
			prettyPrint: true;
			nextPut: objects ].
	^ (NeoJSONReader on: output readStream) 
		next! !

TestCase subclass: #NeoJSONBenchmarkTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Tests'!

!NeoJSONBenchmarkTests commentStamp: 'SvenVanCaekenberghe 6/5/2012 10:05' prior: 0!
I am NeoJSONBenchmarkTests.

This test is obviously dependent on hardware, but the margin is quite high.!

!NeoJSONBenchmarkTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/6/2012 15:11'!
testPrettyPrinted
	| benchmark |
	benchmark := NeoJSONBenchmark new.
	self
		assert: benchmark writePrettyPrintedRead
		equals: benchmark read.
	"Make sure that we can pretty print and parse at least 5k 5-element maps per second"
	self 
		should: [ benchmark writePrettyPrintedRead ]
		notTakeMoreThanMilliseconds: 200! !

!NeoJSONBenchmarkTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/5/2012 10:04'!
testRead
	| benchmark |
	benchmark := NeoJSONBenchmark new.
	"Make sure that we can parse at least 10k 5-element maps per second"
	self 
		should: [ benchmark read ]
		notTakeMoreThanMilliseconds: 100! !

!NeoJSONBenchmarkTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/5/2012 10:04'!
testWrite
	| benchmark |
	benchmark := NeoJSONBenchmark new.
	"Make sure that we can render at least 10k 5-element maps per second"
	self 
		should: [ benchmark write ]
		notTakeMoreThanMilliseconds: 100! !

TestCase subclass: #NeoJSONReaderTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Tests'!

!NeoJSONReaderTests commentStamp: 'SvenVanCaekenberghe 6/2/2012 14:02' prior: 0!
I am NeoJSONReaderTests, a test suite for NeoJSONReader.!

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/2/2012 14:17'!
testBooleans
	self assert: (NeoJSONReader on: 'true' readStream) next equals: true.
	self assert: (NeoJSONReader on: 'false' readStream) next equals: false.! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/2/2012 21:49'!
testFloats
	| reader |
	reader := [ :string |
		(NeoJSONReader on: string readStream) next ].
	self assert: ((reader value: '123.0') closeTo: 123.0).
	self assert: ((reader value: '-123.0') closeTo: -123.0).
	self assert: ((reader value: '123.0e0') closeTo: 123.0).
	self assert: ((reader value: '123e6') closeTo: 123e6).
	self assert: ((reader value: '123e+6') closeTo: 123e6).
	self assert: ((reader value: '123e-6') closeTo: 123e-6).
	self assert: ((reader value: '123E6') closeTo: 123e6).
	self assert: ((reader value: '123E+6') closeTo: 123e6).
	self assert: ((reader value: '123E-6') closeTo: 123e-6).
	self assert: ((reader value: '-123e6') closeTo: -123e6).
	self assert: ((reader value: '-123e-6') closeTo: -123e-6).
	self assert: ((reader value: '-123E6') closeTo: -123e6).
	self assert: ((reader value: '-123E-6') closeTo: -123e-6).
	self assert: ((reader value: '3.14159') closeTo: Float pi).
	self assert: ((reader value: '0.123e6') closeTo: 0.123e6).
	self assert: ((reader value: '0.123e-6') closeTo: 0.123e-6).
	self assert: ((reader value: '-0.123E6') closeTo: -0.123e6).
	self assert: ((reader value: '-0.123E-6') closeTo: -0.123e-6).
! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/2/2012 14:07'!
testIntegers
	| reader |
	reader := [ :string |
		(NeoJSONReader on: string readStream) next ].
	self assert: (reader value: '123') equals: 123.
	self assert: (reader value: '00123') equals: 123.
	self assert: (reader value: '-123') equals: -123.
	self assert: (reader value: '-0123') equals: -123.
	self assert: (reader value: '0') equals: 0.
	self assert: (reader value: '00') equals: 0.
	self assert: (reader value: ' 123	') equals: 123.
! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/2/2012 16:19'!
testLists
	| reader |
	reader := [ :string |
		(NeoJSONReader on: string readStream) next ].
	self assert: (reader value: '[1,2,3]') equals: #(1 2 3).
	self assert: (reader value: '[]') equals: #().
	self assert: (reader value: ' [ 1 , 2 , 3 ] ') equals: #(1 2 3).
	self assert: (reader value: ' [ ] ') equals: #().
! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/2/2012 16:24'!
testMaps
	| reader |
	reader := [ :string |
		(NeoJSONReader on: string readStream) next ].
	self assert: (reader value: '{"x":1,"y":2}') equals: (Dictionary newFromPairs: #( 'x' 1 'y' 2)).
	self assert: (reader value: '{}') equals: Dictionary new.
	self assert: (reader value: ' { "x" : 1 , "y" : 2 } ' ) equals: (Dictionary newFromPairs: #( 'x' 1 'y' 2)).
	self assert: (reader value: ' { } ') equals: Dictionary new.
! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/2/2012 14:18'!
testNull
	self assert: (NeoJSONReader on: 'null' readStream) next equals: nil.
	self assert: (NeoJSONReader on: '	null ' readStream) next equals: nil.! !

!NeoJSONReaderTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/3/2012 19:48'!
testStrings
	| reader |
	reader := [ :string |
		(NeoJSONReader on: string readStream) next ].
	self assert: (reader value: '"foo"') equals: 'foo'.
	self assert: (reader value: '""') equals: ''.
	self assert: (reader value: '"Foo BAR"') equals: 'Foo BAR'.
	self assert: (reader value: '"\u00E9l\u00E8ve en Fran\u00E7ais"') equals: 'lve en Franais'.
	self 
		assert: (reader value: '"\"\\\/\t\r\n\f\b"') 
		equals: (String withAll: { 
					$". $\. $/. Character tab. Character cr. Character lf.  Character newPage. Character backspace }).
! !

TestCase subclass: #NeoJSONWriteReadTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Tests'!

!NeoJSONWriteReadTests commentStamp: 'SvenVanCaekenberghe 6/4/2012 15:34' prior: 0!
I am NeoJSONWriteReadTests.!

NeoJSONWriteReadTests subclass: #NeoJSONWriteReadPrettyPrintedTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Tests'!

!NeoJSONWriteReadPrettyPrintedTests commentStamp: 'SvenVanCaekenberghe 6/5/2012 09:50' prior: 0!
I am NeoJSONWriteReadPrettyPrintedTests.!

!NeoJSONWriteReadPrettyPrintedTests methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/4/2012 15:45'!
writeRead: object
	| json |
	json := String streamContents: [ :stream |
		(NeoJSONWriter on: stream) 
			prettyPrint: true;
			nextPut: object ].
	^ (NeoJSONReader on: json readStream) next ! !

!NeoJSONWriteReadTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/4/2012 15:51'!
testFloats
	| objects |
	objects := { Float pi . 1.0 . -1.0 . 0.0 . 1.5e6 . -1.5e6 . 1.5e-6 . -1.5e-6 }.
	(self writeRead: objects) 
		with: objects 
		do: [ :first :second | self assert: (first closeTo: second) ]! !

!NeoJSONWriteReadTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/4/2012 15:46'!
testIntegers
	| objects |
	objects := { 1. -1. 0. 123467890. -1234567890 }.
	self 
		assert: (self writeRead: objects)
		equals: objects! !

!NeoJSONWriteReadTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/4/2012 15:51'!
testLists
	| objects |
	objects := { #(1 2 3). #(). #( 'foo' 'bar') }.
	self 
		assert: (self writeRead: objects)
		equals: objects! !

!NeoJSONWriteReadTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/4/2012 15:53'!
testMaps
	| objects |
	objects := { 
		Dictionary new.
		Dictionary with: 'x' -> 1.
		Dictionary newFromPairs: #( 'foo' true 'bar' false 'zero' 0 'null' nil 'string' 'Hello World!!' ) }.
	self 
		assert: (self writeRead: objects)
		equals: objects! !

!NeoJSONWriteReadTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/4/2012 15:46'!
testSpecials
	| objects |
	objects := { true. false. nil }.
	self 
		assert: (self writeRead: objects)
		equals: objects! !

!NeoJSONWriteReadTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/4/2012 15:47'!
testStrings
	| objects |
	objects := { 'foo'. 'Foo BAR'. ''. '	\/'''. 'lve en Franais' }.
	self 
		assert: (self writeRead: objects)
		equals: objects! !

!NeoJSONWriteReadTests methodsFor: 'private' stamp: 'SvenVanCaekenberghe 6/4/2012 15:36'!
writeRead: object
	| json |
	json := String streamContents: [ :stream |
		(NeoJSONWriter on: stream) nextPut: object ].
	^ (NeoJSONReader on: json readStream) next ! !

TestCase subclass: #NeoJSONWriterTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Neo-JSON-Tests'!

!NeoJSONWriterTests commentStamp: 'SvenVanCaekenberghe 6/2/2012 22:20' prior: 0!
I am NeoJSONWriterTests, a test suite for NeoJSONWriter.!

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/3/2012 19:56'!
testBooleans
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) nextPut: object ] ].
	self assert: (writer value: true) equals: 'true'.
	self assert: (writer value: false) equals: 'false'.! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/2/2012 22:24'!
testFloats
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) nextPut: object ] ].
	self assert: (writer value: 123.0) equals: '123.0'.
	self assert: (writer value: -123.0) equals: '-123.0'.
	self assert: (writer value: 0.0) equals: '0.0'.! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/2/2012 22:24'!
testIntegers
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) nextPut: object ] ].
	self assert: (writer value: 123) equals: '123'.
	self assert: (writer value: -123) equals: '-123'.
	self assert: (writer value: 0) equals: '0'.! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/3/2012 19:39'!
testLists
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) nextPut: object ] ].
	self assert: (writer value: #(1 2 3)) equals: '[1,2,3]'.
	self assert: (writer value: #()) equals: '[]'.
! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/3/2012 19:40'!
testMaps
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) nextPut: object ] ].
	self assert: (writer value: (Dictionary newFromPairs: #( 'x' 1 'y' 2))) equals: '{"x":1,"y":2}'.
	self assert: (writer value: Dictionary new) equals: '{}'.
! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/2/2012 22:23'!
testNull
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) nextPut: object ] ].
	self assert: (writer value: nil) equals: 'null'! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/6/2012 16:03'!
testPoint
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) nextPut: object ] ].
	self should: [ writer value: 1@2 ] raise: Error! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/6/2012 17:28'!
testPoint1
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) 
				mappingFor: Point do: [ :mapping |
					mapping mapProperty: 'x' getter: [ :point | point x ] setter: [ :point :value | point x: value ].
					mapping mapProperty: 'y' getter: [ :point | point y ] setter: [ :point :value | point y: value ]. ];
				nextPut: object ] ].
	self assert: (writer value: 1@2) equals: '{"x":1,"y":2}'.
	self assert: (writer value: Point new) equals: '{}'! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/6/2012 17:29'!
testPoint2
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) 
				mapAllInstVarsFor: Point;
				nextPut: object ] ].
	self assert: (writer value: 1@2) equals: '{"x":1,"y":2}'.
	self assert: (writer value: Point new) equals: '{}'! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/6/2012 17:29'!
testPoint3
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) 
				mappingFor: Point do: [ :mapping |
					mapping mapInstVars: #(x y) ];
				nextPut: object ] ].
	self assert: (writer value: 1@2) equals: '{"x":1,"y":2}'.
	self assert: (writer value: Point new) equals: '{}'! !

!NeoJSONWriterTests methodsFor: 'testing' stamp: 'SvenVanCaekenberghe 6/3/2012 19:49'!
testStrings
	| writer |
	writer := [ :object | 
		String streamContents: [ :stream |
			(NeoJSONWriter on: stream) nextPut: object ] ].
	self assert: (writer value: 'foo') equals: '"foo"'.
	self assert: (writer value: 'Foo BAR') equals: '"Foo BAR"'.
	self assert: (writer value: '') equals: '""'.
	self
		assert: (writer value: (String withAll: { 
					$". $\. $/. Character tab. Character cr. Character lf.  Character newPage. Character backspace }))
		equals: '"\"\\\/\t\r\n\f\b"'

! !
